///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	ipcomm.h
 * @author	Cal(it)2
 * @author	Anthony Nwokafor
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

#ifndef _P25_COMM_IPCOMM_H
#define _P25_COMM_IPCOMM_H

#include <sys/un.h>

namespace P25 {
namespace Comm {
/**
 * @brief	IP Comminications class definitions.
 */
class IPComm
{
public:
	/**
	 * @brief Constructor for IPComm class.
	 */
	IPComm(const char* skt_path);

	/**
	 * @brief Destructor for IPComm class.
	 */
	~IPComm();

	/**
	 * @brief Send data to socket.
	 */
	int send(char *send_data, int len);

	/**
	 * @brief Receive data from socket.
	 */
	int receive(char *recv_data, int len);

	/**
	 * @brief Initiate a connection to a socket based on Tx or Rx mode.
	 */
	int start_connection(int connType);

	/** 
	 * @brief Terminate a connection from a socket.
	 */
	void stop_connection();

	/** @brief Constant for Receive mode */
	static const int RXMode = 1;
	/** @brief Constant for Transmit mode */
	static const int TXMode = 2;
private:
	
	/**
	 * @brief
	 */
	struct sockaddr_un addr;
	
	/**
	 * @brief 
	 */
	char socket_path[255];
	
	/**
	 * @brief
	 */
	int fdSock;

	/**
	 * @brief 
	 */
	int mConnType;
};

}
}

#endif /* _P25_COMM_IPCOMM_H */
