///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	udplib.c
 * @author	Cal(it)2
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

/* udplib.c */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "p25/util/logger.h"

int start_udp_server(int port)
{
    struct sockaddr_in s_addr;
    int sock = 0;
    int val = 1;

	if ((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
		perror("Could not open Socket!");
		exit(1);
	}

    if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&val,sizeof(val)) == -1) {
		perror("Could not set Setsockopt (SO_REUSEADDR)!");
		exit(1);
	}

    //fcntl(sock,F_SETFL,FNDELAY);

	bzero(&(s_addr),sizeof(s_addr));
	s_addr.sin_family = AF_INET;
	s_addr.sin_port = htons(port);
	s_addr.sin_addr.s_addr = htonl(INADDR_ANY);

	if (bind(sock, (struct sockaddr *)&s_addr, sizeof(struct sockaddr)) == -1) {
		perror("Unable to bind to port");
		exit(1);
	}

    return sock;
} 

void stop_udp_server(int sock)
{
    int res;
    res = close(sock);
    if (!res) sock = 0;
}

int read_udp_server_stream(struct sockaddr_in *p_addr, int sock, char *recv_data, int len)
{
    int bytes_read;
    socklen_t plen = sizeof(struct sockaddr);

    if (sock) {
        bytes_read = recvfrom(sock, recv_data, len, 0, 
                    (struct sockaddr *)p_addr, &plen);
        return bytes_read;
    }

    return 0;
}

int start_udp_client(struct sockaddr_in *p_addr, char *ip, int port)
{
    int sock;

	if ((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
		perror("Could not open Socket!");
		exit(1);
	}

	bzero(p_addr,sizeof(struct sockaddr_in));
    (*p_addr).sin_port = htons(port);
    (*p_addr).sin_family = AF_INET;
    if (inet_aton(ip,&((*p_addr).sin_addr)) == 0) {;
		perror("inet_aton() failed!\n");
		exit(1);
	}

    return sock;
}

int stop_udp_client(int sock)
{
    return close(sock);
}

int send_udp_client_data(struct sockaddr_in *p_addr, int sock, char *send_data, int len)
{
	return sendto(sock, send_data, len, 0, (struct sockaddr *)p_addr, 
                sizeof(struct sockaddr));
}
