///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	intfc.hpp
 * @brief	contains the declaration of the dsp::intfc namespace. This
 *			class defines the interface between the DSP module and the GPP and
 *     		Radio modules of the P25 SDR implementation.
 *
 * @author	Cal(it)2
 * @author  Sivaramakrishnan Chandrasekaran <bscrk29@gmail.com>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
///////////////////////////////////////////////////////////////////////////////

#ifndef _INTFC_H
#define _INTFC_H

/**
 * @brief dsp::intfc function declarations
 */
namespace dsp{

namespace intfc {

/**
 * @brief Inititalizes the interface channels
 */
int intfc_init(const char*, const char*,const char*, const char*);


//Not used. To be removed
int uhdi_tx_en(int en);
int uhdi_rx_en(int en);

/**
 * @brief Read data (bits) from the GPP module through a socket interface.
 */
int read_gppi(char *buf, int len);

/**
 * @brief Write data (bits) to the GPP module through a socket interface.
 */
int write_gppi(char *buf, int len);

//Not used. To be removed
int read_gppi_reset(void);
int write_gppi_flush(void);

/**
 * @brief Read data (stream of floats) from the radio (UHD) module through a FIFO pipe.
 */
int read_uhdi(char*data, int len);


/**
 * @brief Write data (stream of floats) to the radio (UHD) module through a FIFO pipe.
 */
int write_uhdi(char *data, int len);

//Not used. To be removed
int read_uhdi_reset(void);
int write_uhdi_flush(void);


}//end namespace intfc

}//end namespace dsp

#endif
