#include <cstdlib>
#include <cstdio>
#include <string.h>

#include "filter.hpp"

/*
 * fir_alloc - initialize a FIR filter
 *
 * coef: the coefficiants of the FIR filter
 * n: 	 length of the coefficiants
 *
 * return a pointer to a fir structure, or NULL if failed
 */
struct fir *fir_alloc(float *coef, int n)
{
	struct fir *f;

	f = (struct fir *)malloc(sizeof(struct fir) + 2*n*sizeof(float));
	if (!f) {
		printf("fir_alloc malloc failed\n");
		return 0;
	}

	f->order = n;
	f->coef = (float *)(f + 1);
	f->regs = f->coef + n;

	memcpy(f->coef, coef, n*sizeof(float));
	memset(f->regs, 0, n*sizeof(float));

	return f;
}

/*
 * fir_free - destroy a FIR filter
 */
void fir_free(struct fir *f)
{
	free(f);
}

/*
 * fir_reset - set the FIR coefficients to zeros
 */
void fir_reset(struct fir *f)
{
	memset(f->regs, 0, f->order*sizeof(float));
}

/*
 * fir_seq - filter a data sequence
 *
 * f: the FIR filter
 * x: the current value in the data sequence
 *
 * return the current filter output
 */
float fir_seq(struct fir *f, float x)
{
	int i;
	float sum;

	/* shift registers */
	for (i = f->order-1; i >= 1; i--)
		f->regs[i] = f->regs[i-1];
	f->regs[0] = x;

	/* compute sum */
	for (i = 0, sum = 0; i < f->order; i++)
		sum += f->coef[i] * f->regs[i];

	return sum;
}
