//-------------------------------------------------------------------------
/**
 * @file 	receiver.h
 * @brief	Receiver Class Definition
 *
 * @author	Cal(it)2
 * @author  Anthony Nwokafor <aanwokafor@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
//-------------------------------------------------------------------------
#ifndef _RECEIVER_H
#define _RECEIVER_H

#include <pthread.h>
#include "p25/config.h"
#include "controller.h"
#include "vocoder.h"
#include "ui.h"

class Receiver {
private:
	/**
	 * @brief 	Tracks the frame number for the current frame
	 *  		being processed by the Receiver
	 */
	int frame_id;

	/**
	 * @brief 	Tracks the number of bits pulled from the receiver bit stream
	 * @details	It is used in computing the locations of the Status bits (SS)
	 * 			in the receiver bit stream.
	 * @see	Receiver::get_bits
	 */
	int rx_bit_count; //used to be static


	/** @brief */
	int frameIsMine(void);

	/** @brief */
	void play_vcw(char* vcw, int len);

	/** @brief */
	int nid_dec(char* nid_cw, char* duid);

	/** @brief */
	void lsd_dec(char* lsd_cw, char* lsd);

	/** @brief */
	void lc_deformat(struct bit_fifo *lc);

	/** @brief */
	int lcw_dec(char* lcw_cw);

	/** @brief */
	int esw_dec(char* esw_cw);

	/** @brief */
	int hdu_dec(char* hdu);

	/** @brief */
	void ldu_cw_split(char* ldu, int len, char* vc, char* lsd, char* lc);

	/** @brief */
	void ldu_dec(char* ldu1, char type);

	/** @brief */
	void ldu2_dec(char* ldu2);

	/** @brief */
	void tdu_dec(char* tdu);

	/** @brief */
	int tdu_lcw_dec(char* tdu);

	/** @brief */
	int check_fs(char* bits);

	/** @brief gets the specified number of bits from the receiver bit stream */
	int get_bits(char* bits, int len);

	/** @brief */
	void initialize();

	/** @brief */
	void shutdown();

	/** @brief */
	Controller* mController;

	/** @brief */
	Vocoder* mVocoder;

	/** @brief */
	UserInterface* my_userintf;

	/** @brief */
	pthread_t rxThread;

public:

	/** @brief */
	Receiver(Controller* mController, Vocoder* mVocoder,
			UserInterface* my_userintf);

	/** @brief T*/
	void run_rx();

	/** @brief Waits for the Receiver thread to end */
	void join();

	/** @brief Starts Receiver thread */
	static void* start_thread(void *obj);
};

#endif
