/****************************************************************************
Copyright 2007 Virginia Polytechnic Institute and State University
This file is part of the OSSIE __COMP_NAME__.
OSSIE __COMP_NAME__ is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
"(at your option) any later version
OSSIE __COMP_NAME__ is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with OSSIE __COMP_NAME__; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef ENCRYPTIONDEVICE_IMPL_H
#define ENCRYPTIONDEVICE_IMPL_H

#include <stdlib.h>
#include <openssl/des.h>
#include <semaphore.h>

#include "ossie/cf.h"
#include "ossie/PortTypes.h"
#include "ossie/Resource_impl.h"
#include "ossie/debug.h"

#include "standardinterfaces/realChar.h"
#include "standardinterfaces/realChar_u.h"
#include "standardinterfaces/realChar_p.h"

#include "p25/config.h"
#include "p25/constants.h"
#include "p25/controller.h"

extern "C" {
#include "p25/format/byte_fifo.h"
}


/** \brief
 *
 *
 */
class EncryptionDevice_i : public virtual Resource_impl
{
  public:
    /** @brief Initializing constructor. */
    EncryptionDevice_i(const char *uuid, omni_condition *sem);

    /** @brief Destructor. */
    ~EncryptionDevice_i(void);

    /** @brief function for omni thread. */
    static void crypt_Run( void * data );

    /** @brief function for omni thread. */
    static void Run( void * data );

    /** @brief function for omni thread. */
    static void vcw_Run( void * data );

    /** @brief function for omni thread. */
    static void rx_frame_Run( void * data );

    /** @brief Static function for omni thread. */
    static void tx_frame_Run( void * data );

    /** @brief Static function for omni thread. */
    int push_crypto_pktizer(char *, int);

    /** @brief Static function for omni thread. */
    int push_crypto_vocoder(char *, int);

    /** @brief Static function for omni thread. */
    static void vcw_out_Run( void * data );

    ///
    void start() throw (CF::Resource::StartError, CORBA::SystemException);

    ///
    void stop() throw (CF::Resource::StopError, CORBA::SystemException);

    ///
    CORBA::Object_ptr getPort( const char* portName )
        throw (CF::PortSupplier::UnknownPort, CORBA::SystemException);

    ///
    void releaseObject() throw (CF::LifeCycle::ReleaseError, CORBA::SystemException);

    ///
    void initialize() throw (CF::LifeCycle::InitializeError, CORBA::SystemException);

	/** @brief Query properties from .prf.xml. */
	void query(CF::Properties&)
		throw (CORBA::SystemException,
				CF::UnknownProperties);

    /** @brief Configures properties read from .prf.xml. */
    void configure(const CF::Properties&)
        throw (CORBA::SystemException,
            CF::PropertySet::InvalidConfiguration,
            CF::PropertySet::PartialConfiguration);


  private:
    /** @brief Disallow default constructor. */
    EncryptionDevice_i();

    /** @brief Disallow copy constructor. */
    EncryptionDevice_i(EncryptionDevice_i&);

    /** @brief Main signal processing method. */
    void ProcessData();
	bool continue_processing();
	volatile bool thread_started;
	omni_mutex processing_mutex;

	/** @brief VCW signal processing method. */
	void vcw_ProcessData();
	bool vcw_continue_processing();
	volatile bool vcw_thread_started;
	omni_mutex vcw_processing_mutex;

	/** @brief Framing signal processing method. */
	void rx_frame_ProcessData();
	bool rx_frame_continue_processing();
	volatile bool rx_frame_thread_started;
	omni_mutex rx_frame_processing_mutex;

	/** @brief Framing signal processing method. */
	void tx_frame_ProcessData();
	bool tx_frame_continue_processing();
	volatile bool tx_frame_thread_started;
	omni_mutex tx_frame_processing_mutex;

	/** @brief Cryptography signal processing method. */
   	void crypt_ProcessData();
	bool crypt_continue_processing();
	volatile bool crypt_thread_started;
	omni_mutex crypt_processing_mutex;

	/** @brief Cryptography signal processing method. */
   	void vcw_out_ProcessData();
	bool vcw_out_continue_processing();
	volatile bool vcw_out_thread_started;
	omni_mutex vcw_out_processing_mutex;

   
    omni_condition *component_running;  	///< for component shutdown
    omni_thread *processing_thread;     	///< for component writer function
    omni_thread *vcw_processing_thread; 	///< for receiving and filling vcw fifo
    omni_thread *rx_frame_processing_thread;///< for receiving and filling framer fifo in Rx mode
    omni_thread *tx_frame_processing_thread;///< for recieving and filling framer fifo in Tx mode
    omni_thread *crypt_processing_thread;  	///< for actual cryptography
    omni_thread *vcw_out_processing_thread; ///< for pushing vcws and vcws only
    	

    
    // list components provides and uses ports
    standardInterfaces_i::realChar_p *enc_in;	///< Incoming data for encryption
    standardInterfaces_i::realChar_u *enc_out;	///< Outgoing data from encryption
    standardInterfaces_i::realChar_p *tx_in;	///< Port for incoming data from TX device
    standardInterfaces_i::realChar_u *tx_out;	///< Port for outgoing data to TX device
    standardInterfaces_i::realChar_p *rx_in;	///< Port for incoming data from RX device
    standardInterfaces_i::realChar_u *rx_out;	///< Port for outgoing data to RX device


	/** @brief	Pointer to Controller object */
	Controller 		*mController;

	/**
	 * @brief Variable shared for synchronization of User Interface changes.
	 */
	UInterface m_ui;

	/** @brief State variable for radio switching activity. */
	bool radio_activity_switch;
	/** @brief Sends data from encryption device to packetizer devices. */
	void sendbuf_frame_data (short dev, short hdr, char *data, int len);

};
#endif
