/****************************************************************************
Copyright 2007 Virginia Polytechnic Institute and State University
This file is part of the OSSIE __COMP_NAME__.
OSSIE __COMP_NAME__ is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
"(at your option) any later version
OSSIE __COMP_NAME__ is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with OSSIE __COMP_NAME__; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef OVEROVOCODERDEVICE_IMPL_H
#define OVEROVOCODERDEVICE_IMPL_H

#include <stdlib.h>
#include "ossie/cf.h"
#include "ossie/PortTypes.h"
#include "ossie/Resource_impl.h"
#include "ossie/debug.h"

#include "standardinterfaces/realChar.h"
#include "standardinterfaces/complexShort.h"
#include "standardinterfaces/realChar_u.h"
#include "standardinterfaces/complexShort_u.h"
#include "standardinterfaces/realChar_p.h"
#include "standardinterfaces/complexShort_p.h"

/* user includes */
#include "p25/config.h"
#include "p25/controller.h"
#include "p25/format/packets.h"
#include "VcwBuffers.h"
#include "host_gpp_intf.h"

extern "C" {
#include "p25/format/bit_fifo.h"
}

/** \brief
 *
 *
 */
class OveroVocoderDevice_i : public virtual Resource_impl
{
  public:
    /// Initializing constructor
    OveroVocoderDevice_i(const char *uuid, omni_condition *sem);

    /// Destructor
    ~OveroVocoderDevice_i(void);

    /// Static function for omni thread
    static void host_voc_Run( void * data );

    /// Static function for omni thread
    static void rx_in_Run( void * data );

    /// Static function for omni thread
    static void tx_in_Run( void * data );

    /// Static function for omni thread
    static void voc_host_Run( void * data );

    /// Static function for omni thread
    static void encrypt_out_Run( void * data );

    /// Static function for omni thread
    static void encrypt_in_Run( void * data );

    ///
    void start() throw (CF::Resource::StartError, CORBA::SystemException);

    ///
    void stop() throw (CF::Resource::StopError, CORBA::SystemException);

    ///
    CORBA::Object_ptr getPort( const char* portName )
        throw (CF::PortSupplier::UnknownPort, CORBA::SystemException);

    ///
    void releaseObject() throw (CF::LifeCycle::ReleaseError, CORBA::SystemException);

    ///
    void initialize() throw (CF::LifeCycle::InitializeError, CORBA::SystemException);

	/// Query properties from .prf.xml
	void query(CF::Properties&)
		throw (CORBA::SystemException,
				CF::UnknownProperties);

    /// Configures properties read from .prf.xml
    void configure(const CF::Properties&)
        throw (CORBA::SystemException,
            CF::PropertySet::InvalidConfiguration,
            CF::PropertySet::PartialConfiguration);


  private:
    /// Disallow default constructor
    OveroVocoderDevice_i();

    /// Disallow copy constructor
    OveroVocoderDevice_i(OveroVocoderDevice_i&);

    /// Host -> ARM signal processing thread
    void host_voc_ProcessData();
	bool host_voc_continue_processing();
	volatile bool host_voc_thread_started;
	omni_mutex host_voc_processing_mutex;

    /// Input signal processing method
    void rx_in_ProcessData();
	bool rx_in_continue_processing();
	volatile bool rx_in_thread_started;
	omni_mutex rx_in_processing_mutex;

	// UI signal processing method
    void tx_in_ProcessData();
	bool tx_in_continue_processing();
	volatile bool tx_in_thread_started;
	omni_mutex tx_in_processing_mutex;

	// ARM signal processing method
    void voc_host_ProcessData();
	bool voc_host_continue_processing();
	volatile bool voc_host_thread_started;
	omni_mutex voc_host_processing_mutex;

	// Encryption signal processing method (to enc. device)
    void encrypt_out_ProcessData();
	bool encrypt_out_continue_processing();
	volatile bool encrypt_out_thread_started;
	omni_mutex encrypt_out_processing_mutex;

	// Encryption signal processing method (from enc. device)
    void encrypt_in_ProcessData();
	bool encrypt_in_continue_processing();
	volatile bool encrypt_in_thread_started;
	omni_mutex encrypt_in_processing_mutex;
   
    omni_condition *component_running;  ///< for component shutdown
    omni_thread *host_voc_processing_thread;     ///< for component writer function
    omni_thread *rx_in_processing_thread;     ///< for component writer function
    omni_thread *tx_in_processing_thread;     ///< for component writer function
    omni_thread *voc_host_processing_thread;     ///< for component writer function
    omni_thread *encrypt_out_processing_thread;
    omni_thread *encrypt_in_processing_thread;

    // list components provides and uses ports
	standardInterfaces_i::realChar_p *tx_in_data;
	standardInterfaces_i::realChar_u *tx_out_data;
	standardInterfaces_i::realChar_p *rx_in_data;
	standardInterfaces_i::realChar_u *rx_out_data;
	standardInterfaces_i::complexShort_p *au_dataIn;
	standardInterfaces_i::complexShort_u *au_dataOut;
    standardInterfaces_i::realChar_u *encrypt_out;
    standardInterfaces_i::realChar_p *encrypt_in;

    /* user variables */

	/** @brief	Pointer to Controller object */
    Controller 		*mController;

    /** @brief	Pointer to Vocoder object (for playback buffer) */
    VcwBuffers			*tVc;

	/**
	 * @details Variable shared for synchronization of User Interface changes.
	 */
    UInterface m_ui;

    int end_instream;
    int end_outstream;
    int end_out_encstream;
    int end_in_encstream;
    int echanbits;

    FILE *testfile_enc;

    HostGPPInterface *hg;
    HostGPPInterface *hg_enc;

	/** @brief	Bit FIFO buffer that holds data waiting to be encrypted. */
	struct	bit_fifo 	*orig_bf;

	/** @brief	Bit FIFO buffer that holds data after encryption. */
	struct	bit_fifo 	*enc_bf;

	/** @brief */
	pthread_mutex_t		orig_bf_m;

	/** @brief */
	pthread_mutex_t		enc_bf_m;

	sem_t	sem_arm_processdata_start;
	sem_t	sem_processdata_start;

	sem_t 	sem_arm_processdata_done;
	sem_t	sem_processdata_done;

	bool ambe_stop_sent;


	unsigned char rx_algid;
	unsigned char tx_algid;

	void sendbuf_vocoder_tx(short hdr, char *data = NULL, int len = 0);
	void sendbuf_vocoder_crypt(short hdr, char *data = NULL, int len = 0);

};
#endif
