//-------------------------------------------------------------------------
/**
 * @file 	VcwBuffers.h
 * @brief	VcwBuffers Class Definition
 *
 * @author	Cal(it)2
 * @author  Justyn Bell <jwbell@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 OSSIE/Linux Implementation
 * @endverbatim
 */
//-------------------------------------------------------------------------
#ifndef _VCWBUFFERS_H
#define _VCWBUFFERS_H

#include <semaphore.h>

/**
 * @brief Class for handling manipulation of VCW through bit FIFOs.
 * 
 */
class VcwBuffers
{
public:

	/** @brief Contructor for vcw buffer fifos. */
	VcwBuffers(void);

	/** @brief Destructor for vcw buffer fifos. */
	~VcwBuffers(void);

	/** @brief Puts VCW data into FIFO buffer. */
	void	put_in_pb_buf(char* vcw, int len);

	/** @brief Gets VCW data from FIFO buffer. */
	int		get_from_pb_buf(char* vcw, int len);

	/** @brief Flushes contents of VCW buffers. */
	void	flush_buffer();

	/** @brief Initializes VCW buffers. */
	void	initialize();

	/** @brief Shuts down VCW buffers. */
	void	shutdown();

private:
	
	/** @brief	Bit FIFO buffer that holds received voice codewords waiting to be played back. */
	struct	bit_fifo 	*pb_bf;

	/** @brief Used for FIFO producer<->consumer model. */
	pthread_mutex_t		vc_pb_bf_m;

	/** @brief Used for FIFO producer<->consumer model. */
	sem_t	sem_pb_bf_empty;
	/** @brief Used for FIFO producer<->consumer model. */
	sem_t	sem_pb_bf_notempty;
	/** @brief Used for flushing buffers. */
	sem_t 	sem_pb_bf_flush;

	/** @brief Used for flushing buffers. */
	char	pb_bf_accept_less;
};

#endif
