/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef _CNTRL_TYPES_H
#define _CNTRL_TYPES_H

/** @brief Length of AMBE-encoded buffer in bytes. */
#define E_BUFF_LEN      288

/** @brief Length of buffer in bytes. */
#define BUFF_LEN  		320

/** @brief Length used for PCM sampling/playback - each sample is 16 bits long. */
#define S_BUFF_LEN	BUFF_LEN / 2

/**
 * @brief Header control messages for OSSIE-->Overo GPP packets.
 */
enum imbe_ctrl_t {
	IMBE_UNKNOWN,				///< Unknown packet header.  Returned if an unknown packet is read.
	IMBE_DATA,					///< Indicates payload of packet contains data.
	IMBE_ENCODE_START,			///< Start encode message for Overo GPP (no encryption).
	IMBE_ENCR_ENCODE_START,		///< Start encode message for Overo GPP (requesting encryption).
	IMBE_ENCODE_STOP,			///< Stop encoding message for Overo GPP.
	IMBE_DECODE_START,			///< Start decode message for Overo GPP (no encryption).
	IMBE_DECR_DECODE_START,		///< Start decode message for Overo GPP (requesting encryption).
	IMBE_DECODE_STOP,			///< Stop decoding message for Overo GPP.
	I_SOCK_DISCONNECT,			///< If no data is read, this message is returned.
	I_SOCK_ERROR				///< If there is an error trying to read data, this message is returned.
};

/**
 * @brief Header control messages for Overo GPP-->OSSIE packets.
 */
enum host_ctrl_t {
	HOST_UNKNOWN,				///< Unknown packet header.  Returned if an unknown packet is read.
	HOST_STREAM_DATA,			///< Indicates payload of packet contains data.
	HOST_START_STREAM,			///< Start processing message for OSSIE OveroVocoder component.
	HOST_STOP_STREAM,			///< Stop processing message for OSSIE OveroVocoder component.
	H_SOCK_DISCONNECT,			///< Not implemented.
	H_SOCK_ERROR				///< Not implemented.
};

/** @brief Host -> ARM (forward path) interface packet format for Encode mode (BUFF_LEN). */
struct e_host_gpp_pkt {
	enum imbe_ctrl_t 	ctrl;					///< Packet header.
	char 				payload[BUFF_LEN];		///< Packet payload.
};

/** @brief ARM -> Host (return path) interface packet format for Encode mode (E_BUFF_LEN). */
struct e_gpp_host_pkt {
	enum host_ctrl_t	ctrl;					///< Packet header.
	char				payload[E_BUFF_LEN];	///< Packet payload.
};

/** @brief Host -> ARM (forward path) interface packet format for Decode mode (E_BUFF_LEN). */
struct d_host_gpp_pkt {
	enum imbe_ctrl_t 	ctrl;					///< Packet header.
	char 				payload[E_BUFF_LEN];	///< Packet payload.
};

/** @brief ARM -> Host (return path) interface packet format for Decode mode (BUFF_LEN). */
struct d_gpp_host_pkt {
	enum host_ctrl_t	ctrl;					///< Packet header.
	char				payload[BUFF_LEN];		///< Packet payload.
};

#endif
