#ifndef _P25_CONFIG_H
#define _P25_CONFIG_H

#include "p25/constants.h"

//#define _DEBUG

/** @brief Header for packets sent between Java GUI and User Interface Device for control information. */
const char             p25_cntrl_header[2] = {0x0A,0xA0};
/** @brief Header for packets sent between Java GUI and User Interface Device for scanning. */
const char			   p25_scan_header[2] = {0x0B,0xB0};

/** @brief State of scanner. */
const char			   G_SCAN_CONTINUE = 0x01;
/** @brief State of scanner. */
const char			   G_SCAN_HALT = 0x00;

/** @brief Default Network Access Code. */
const unsigned short   NAC = 0x0293;
/** @brief Default Talk-group ID. */
const unsigned short   TGID = 0x0001;
/** @brief Default Unit ID. */
const unsigned char    UID[3] = {0x00,0x00,0x01};
/** @brief Default Emergency Indicator. */
const unsigned char    EBIT = 0x0;
/** @brief Default Algorithm ID. */
const unsigned char    ALGID = 0x80;
/** @brief Default Rx encryption state. */
const unsigned char    E_STATE = 0x00; 
/** @brief Default Key ID. */
const unsigned short   KID = 0x0000;
/** @brief Default Manufacturer ID. */
const unsigned char    MFID = 0x00;
/** @brief Default Message Indicator. */
const unsigned char    MI[9] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

/** @brief Unit ID designated for group calls to everyone. */
const unsigned char    EVERYONE_UID[3] = {0xFF,0xFF,0xFF};
/** @brief Talk-group ID that includes everyone. */
const unsigned short   EVERYONE_TGID = 0xFFFF;
/** @brief No NAC Mode NAC constant. */
const unsigned short   P25_NO_NAC_MODE = 0x0F7E;



#pragma pack(push,1)
/** @brief Struct used for data passed between GUI and OSSIE.
  * @details GUI sends data one element at a time, so UInterface struct needs to be packed.
  */
typedef struct {
    double          cfreq; 	///< Center frequency
    unsigned short  nac;	///< Network Access Code
    unsigned short  tgId;	///< Talk-group ID
    unsigned short  keyId;	///< Key ID
    unsigned char   mfId;	///< Manufacturer ID
    unsigned char   algId;	///< Algorithm ID
    unsigned char   ebit;	///< Emergency Bit
    unsigned char   unitId[3];	///< Unit ID
    unsigned char   dstId[3];	///< Destination ID
    unsigned char   mi[9];	///< Message Indicator
    unsigned char   tfpath[255];///< Test File path
    unsigned char  	tfmode;	///< Test File Mode
    unsigned char   ptt;	///< Push to talk status
	unsigned char   rxEncState;///< Encryption state for incoming calls
} UInterface;
#pragma pack(pop)

#pragma pack(push,1)
/** @brief Struct used for scanning data passed from GUI to OSSIE.
  * @details GUI sends data one element at a time, so SUInterface struct needs to be packed.
  */
typedef struct {
	unsigned char	state;	///< State of scanner
	unsigned short 	snum;	///< Number of frequencies to scan
	double			freqs[15];	///< List of frequencies
} SUInterface;
#pragma pack(pop)

#pragma pack(push,1)
/** @brief Struct used for scanning data passed from OSSIE to GUI.
  * @details Contains the stop state and also the frequency that it stopped on so that the GUI can be
  *				updated accordingly.
  */
typedef struct {
	unsigned char state;   ///< State of scanner on OSSIE side
	double freq;           ///< Last frequency on OSSIE side that found a valid FS
} GUInterface;
#pragma pack(pop)

/* @brief Radio Control struct. */
typedef struct {
    int    mode;		///<
    int    busy;		///<
} RadioControl;

#endif
