/*
 * ipcomm.c
 *
 *  Created on: Mar 8, 2011
 *      Author: Anthony Nwokafor
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <fcntl.h>

#include "p25/comm/ipcomm.h"
#include "p25/util/logger.h"

#include <errno.h>

namespace P25 {
namespace Comm {

/** @details */
IPComm::IPComm(const char* s_path)
{
    fdSock = 0;
    strcpy(socket_path,s_path);

	bzero(&(addr),sizeof(addr));
	addr.sun_family = AF_UNIX;
	strcpy(addr.sun_path, socket_path);
}

/**
 *  @details Starts a connection based on communicating between processes on the
 *			 same machine via datagram sockets.
 */
int IPComm::start_connection(int connType)
{
    int val = 1;
    mConnType = connType;

	if ((fdSock = socket(PF_UNIX, SOCK_DGRAM, 0)) < 0) {
		perror("Could not open Socket!");
		exit(-1);
	}

	switch (connType) {
	case IPComm::RXMode:
		if (setsockopt(fdSock,SOL_SOCKET,SO_REUSEADDR,&val,sizeof(val)) < 0) {
			perror("Could not set socket option SO_REUSEADDR!");
			exit(-1);
		}

		unlink(socket_path);

		if (bind(fdSock, (struct sockaddr *) &addr, sizeof(struct sockaddr_un)) != 0) {
			perror("Unable to bind to port");
			exit(-1);
		}
		break;
	case IPComm::TXMode:
		// Nothing to do
		break;
	default:
		return -1;
		break;
	}

    return 0;
} 

/**
 * @details
 */
void IPComm::stop_connection()
{
    int res;
    res = close(fdSock);
    if (!res) fdSock = 0;
    unlink(socket_path);
}

/**
 * @details
 */
int IPComm::receive(char* recv_data, int len)
{
	int rcvlen;
	struct sockaddr_un rcv_addr;

    if (fdSock && mConnType == IPComm::RXMode) {
    	rcvlen = sizeof(rcv_addr);
        return recvfrom(fdSock, recv_data, len, 0, (struct sockaddr *) &rcv_addr,
        	(socklen_t *) &rcvlen);
    }
    return -1;
}

/**
 * @details
 */
int IPComm::send(char* send_data, int len)
{
	int res, serror = 0;
	if (mConnType == IPComm::TXMode) {
		res = sendto(fdSock, send_data, len, 0, (struct sockaddr *) &addr,
	                sizeof(struct sockaddr));

		if (res == -1) {
			serror = errno;
			printf("error:  %s\n", strerror(serror));
		}

		return res;
	}
	return -1;
}

} // namespace Comm
} // namespace P25
