/**
 * ============================================================================
 * @file	ticktock.h
 * @brief	This file implements a timer
 * @note	none.
 *
 * @author	Cal(it)2
 * @author	Jeff Cuenco <jcuenco@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 * @endverbatim
 * ============================================================================
 */

#ifndef _TICK_TOCK_H
#define _TICK_TOCK_H

// tick-tock timer macros
#include <sys/time.h>

#define TICK() struct timeval tp1, tp2; gettimeofday(&tp1, NULL); 

#define TICK1() gettimeofday(&tp1, NULL); 

#define _TOCK(fmt, thresh)      {                          \
        long mtime, secs, usecs;                           \
        gettimeofday(&tp2, NULL);                          \
        secs = tp2.tv_sec - tp1.tv_sec;                    \
        usecs = tp2.tv_usec - tp1.tv_usec;                 \
        mtime = ((secs) * 1000 + usecs/1000.0) + 0.5;      \
        if (mtime > thresh)                                \
                INFO("%s time:  %ld ms.\n", fmt, mtime);   \
}

#define TOCK(fmt) _TOCK(fmt, 0)

#endif
