/**
 * ============================================================================
 * @file	ambe_intf.h
 * @brief	This file contains definitions for the processing functions for
 * 			the AMBE Interface library. This file contains modifications from
 * 			the test file provided by DVSI
 * @note	none.
 *
 * @author 	DVSI
 * @author	Cal(it)2
 * @author	Anthony Nwokafor <aanwokafor@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 * @endverbatim
 * ============================================================================
 */

#ifndef AMBE_TEST_H
#define AMBE_TEST_H

#include "common.h"
#include "apcop2.h"
#include "apcop2fec.h"
#include "pcargs.h"
#include "timer.h"

//#define MEASURE_STACK_SIZE
//#define PRINT_TIMING
//#define OVERALL_MAXTIME

/*
 * AMBE Vocoder Interface structure
 */

#ifndef BUILD_REPEATER
typedef short REPENCSTATE; 
typedef short REPDECSTATE;
#define REPENCSTATE_T_SZ (sizeof(REPENCSTATE)/sizeof(short))
#define REPENCSTATE_SZ (sizeof(REPENCSTATE)/sizeof(short))
#define REPDECSTATE_T_SZ (sizeof(REPDECSTATE)/sizeof(short))
#define REPDECSTATE_SZ (sizeof(REPDECSTATE)/sizeof(short))
#endif


#ifdef BUILD_REPEATER
REPENCSTATE_T *estrep;
REPDECSTATE_T *dstrep;

REPENCSTATE_T f_estrep, s_estrep;
REPDECSTATE_T f_dstrep, s_estrep;
#endif
/*
 * all I/O is in 16-bit words
 */
#define SIZEINT 	2
#define SIZEBIT		1
#define PACKED_BITS_SZ 256 /* bytes */

#define VAD_LEVEL_MIN 	-90
#define VAD_LEVEL 	0
#define VAD_LEVEL_MAX 	3
#define TONE_AMP 	-10
#define TONE_INDEX 	128
#define SD_BITS_MIN 	1
#define SD_BITS 	1
#define SD_BITS_MAX 	8

#define SPEECH_BUF_SZ 2*AMBE_WS_MAX

#define CHAN_BUF_SZ (7200/50)


typedef struct ests_t {
	short 		cmode;
	short 		status;
	short 		wshift;
	short 		bit_steal;
	short 		cni;
	TONE_INFO 	info;
	int 		vad_frames;
	int 		dtx_frames;
	int 		tone_frames;
	int		reset_frames;
	int 		frames;
} ESTS;

typedef struct dsts_t {
	short 		cmode;
	short 		status;
	short 		wshift;
	short 		bit_steal;
	short 		ttl_errs;
	short 		cwerrs1;
	TONE_INFO 	info;
	int 		data_invalid_frames;
	int		erasure_frames;
	int 		silence_frames;
	int 		tone_frames;
	int		reset_frames;
	int 		frames;
} DSTS;

extern cregister volatile unsigned int CSR;
 /*
 * sets the PCC bits in CSR to enable the cache
 */
void enable_cache( void );


extern int _stack[];

/*
 * Note that these functions can be used to measure the amount of
 * stack space used by a particular function. TO use them, you
 * must call clear_stack before you call the function for
 * which you are measuring. Then call the function that you
 * are measuing. After it returns call get_stack_size. clear_stack
 * returns the amount of free stack space (in bytes), at the time
 * that it is called. clear_stack initializes all of the free stack
 * space to a fixed value of 0xaa. The return value from
 * clear_stack should be passed as the argument to get_stack_size.
 * get_stack_size looks to see how many of the bytes on the stack
 * were modified by the function being measured. It does this by
 * counting the number of bytes that are still equal to 0xaa.
 * get_stack_size returns the amount of stack space (in bytes)
 * that were used by the function under test.
 *
 * NOTES:
 * -This test is performed with no interrupts running. If you
 * have an ISR that uses data on the stack this method can not
 * be used, since it momentarily uses up the whole stack
 * (on purpose).
 * -Add a couple bytes to the measured size for safety.
 * "&value_on_stack" should be a good estimate of sp.
 */
#ifdef MEASURE_STACK_SIZE
#define CLEAR_STACK() stack_count = clear_stack()
#define GET_STACK_SIZE_MAX( max ) max = get_stack_size_max( stack_count, max )
#else
/*
 * make them do nothing if MEASURE_STACK_SIZE isn't defined.
 */
#define CLEAR_STACK()
#define GET_STACK_SIZE_MAX( max )
#endif

int clear_stack( void );

/*
 * returns the number of stack locations that are unused. Unused bytes
 * are equal to zero since they were cleared using clear_stack.
 */
int get_stack_size( int stack_before );

/*
 * use this function to compute the maximum of all measurements.
 */
int get_stack_size_max( int stack_before, int max );

void init_coders(AMBE_OPTIONS * ao);

int process_encode_request(AMBE_OPTIONS * ao);
int process_decode_request(AMBE_OPTIONS * ao);

#endif //AMBE_TEST_H
