/** ============================================================================
 *  @file   dsp_ambe_config.h
 *  @path   $(DSPLINK)/dsp/src/samples/dsp_ambe/
 *  @desc   Header file containing configuration for the DSPABME sample.
 *  @ver    1.65.00.03
 *  ============================================================================
 *  ============================================================================
 */


#if !defined (DSP_AMBE_CONFIG_)
#define DSP_AMBE_CONFIG_


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @const  SAMPLE_POOL_ID
 *  @desc   Pool identifier for this application.
 *  ============================================================================
 */
#define SAMPLE_POOL_ID  0

/** ============================================================================
 *  @const  DSPLINK_DRVNAME
 *  @desc   Name of the DSPLINK driver. This is different based on whether the
 *          sample application uses TSKs or SWIs for data transfer.
 *  ============================================================================
 */
#define DSPLINK_DRVNAME  "/dio_dsplink"

/** ============================================================================
 *  @const  CHNL_NAME
 *  @desc   Name of the channel, generated based on the DSPLINK driver
 *          name specified.
 *  ============================================================================
 */
#define CHNL_NAME(drvName)  drvName ## "0"

/** ============================================================================
 *  @const  INPUT_CHANNEL
 *  @desc   Name of the input channel.
 *  ============================================================================
 */
#define INPUT_CHANNEL             DSPLINK_DRVNAME ## "0"

/** ============================================================================
 *  @const  OUTPUT_CHANNEL
 *  @desc   Name of the output channel.
 *  ============================================================================
 */
#define OUTPUT_CHANNEL            DSPLINK_DRVNAME ## "1"

/** ============================================================================
 *  @const  CINPUT_CHANNEL
 *  @desc   Name of the cryptography input channel.
 *  ============================================================================
 */
#define CINPUT_CHANNEL            DSPLINK_DRVNAME ## "2"

/** ============================================================================
 *  @const  COUTPUT_CHANNEL
 *  @desc   Name of the cryptography output channel.
 *  ============================================================================
 */
#define COUTPUT_CHANNEL           DSPLINK_DRVNAME ## "3"

#if defined (CHNL_PCPY_LINK)
/** ============================================================================
 *  @name   NUM_BUF_SIZES
 *  @desc   Number of buffer pools to be configured for the pool.
 *  ============================================================================
 */
#define NUM_BUF_SIZES       1

/** ============================================================================
 *  @name   NUM_BUFFERS
 *  @desc   Number of buffers to be allocated.
 *  ============================================================================
 */
#define NUM_BUFFERS         4

/** ============================================================================
 *  @name   SAMPLEPOOL_open
 *  @desc   Macro over the allocator for opening a pool.
 *  @arg    poolId
 *              Pool id.
 *  @arg    params
 *              Parameters.
 *  @see    SAMPLEPOOL_close
 *  ============================================================================
 */
#define SAMPLEPOOL_open(poolId, params)                        \
        (((POOL->allocators [poolId]).fxns->open)              \
                (&(POOL->allocators [poolId].object),          \
                 params))

/** ============================================================================
 *  @name   SAMPLEPOOL_close
 *  @desc   Macro over the allocator for closing a pool.
 *  @arg    poolId
 *              Pool id.
 *  @see    SAMPLEPOOL_open
 *  ============================================================================
 */
#define SAMPLEPOOL_close(poolId)                         \
        (((POOL->allocators [poolId]).fxns->close)       \
                (POOL->allocators [poolId].object))

/** ============================================================================
 *  @name   POOL
 *  @desc   Configuration variable defined by the POOL component.
 *  ============================================================================
 */
extern POOL_Config  * POOL ;

/** ============================================================================
 *  @name   DSPABME_Pools
 *  @desc   Array of pools.
 *  ============================================================================
 */
extern POOL_Obj DSPABME_Pools [] ;
#endif /* if defined (CHNL_PCPY_LINK) */


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */


#endif /* !defined (DSP_ABME_CONFIG_) */
