#include <stdio.h>
#include <stdlib.h>
#include <alsa/asoundlib.h>
#include "alsa_capture.h"
#include "alsa_avail.h"

#define A_NUM_CHANNELS		1
      
snd_pcm_t *capture_handle;

int pcm_cap_init (char *snd_cap_device, unsigned int snd_samprate)
{
	int err;
	snd_pcm_hw_params_t *hw_params;

	printf("Available hardware:\n");
	alsa_avail();

	
	if ((err = snd_pcm_open (&capture_handle, snd_cap_device, SND_PCM_STREAM_CAPTURE, 0)) < 0) {
		fprintf (stderr, "cannot open audio device %s (%s)\n", 
			 snd_cap_device,
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
		fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_any (capture_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_access (capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
		fprintf (stderr, "cannot set access type (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_format (capture_handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
		fprintf (stderr, "cannot set sample format (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_rate_near (capture_handle, hw_params, &snd_samprate, 0)) < 0) {
		fprintf (stderr, "cannot set sample rate (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_channels (capture_handle, hw_params, A_NUM_CHANNELS)) < 0) {
		fprintf (stderr, "cannot set channel count (%s)\n",
			 snd_strerror (err));

		return err;
	}

	/* Apply HW parameter settings to PCM device */
	if ((err = snd_pcm_hw_params (capture_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot set parameters (%s)\n",
			 snd_strerror (err));

		return err;
	}
	snd_pcm_hw_params_free (hw_params);

	/* Prepare device and return */
	return pcm_capture_prepare();
}
	
int pcm_capture(char *buffer, int length) 
{
	int err;
	
	if ((err = snd_pcm_readi (capture_handle, buffer, length)) != length) {
		fprintf (stderr, "read from audio interface failed (%s)\n",
			snd_strerror (err));

		return err;
	}
	return 0;
}

int pcm_capture_prepare(void)
{
	int err;

	if ((err = snd_pcm_prepare (capture_handle)) < 0) {
		fprintf (stderr, "cannot prepare audio interface for use (%s)\n",
			 snd_strerror (err));

		return err;
	}

	return 0;
}

int pcm_cap_close() 
{
	snd_pcm_close (capture_handle);
	return 0;
}
