#include <stdio.h>
#include <stdlib.h>
#include <alsa/asoundlib.h>
#include "alsa_play.h"
#include "alsa_avail.h"
      
#define A_NUM_CHANNELS		1



snd_pcm_t *playback_handle;

int pcm_play_init(char *snd_play_device, unsigned int snd_samprate)
{
	int err;
	snd_pcm_hw_params_t *hw_params;

	
	printf("Available hardware:\n");
	alsa_avail();

	if ((err = snd_pcm_open (&playback_handle, snd_play_device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
		fprintf (stderr, "cannot open audio device %s (%s)\n", 
			 "default",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
		fprintf (stderr, "cannot allocate hardware parameter structure (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_any (playback_handle, hw_params)) < 0) {
		fprintf (stderr, "cannot initialize hardware parameter structure (%s)\n",
			 snd_strerror (err));

		return err;
	}
/*
	if ((err = snd_pcm_hw_params_set_rate_resample(playback_handle, resample)) < 0) {
		fprintf( stderr, "cannot set resample parameter (%s)\n", 
			snd_strerror (err));

		return err;
	}
*/
	if ((err = snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
		fprintf (stderr, "cannot set access type (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
		fprintf (stderr, "cannot set sample format (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_rate_near (playback_handle, hw_params, &snd_samprate, 0)) < 0) {
		fprintf (stderr, "cannot set sample rate (%s)\n",
			 snd_strerror (err));

		return err;
	}
	if ((err = snd_pcm_hw_params_set_channels (playback_handle, hw_params, A_NUM_CHANNELS)) < 0) {
		fprintf (stderr, "cannot set channel count (%s)\n",
			 snd_strerror (err));

		return err;
	}

        /* Apply HW parameter settings to PCM device */
        if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0) {
                fprintf (stderr, "cannot set parameters (%s)\n",
                         snd_strerror (err));

                return err;
        }
        snd_pcm_hw_params_free (hw_params);

	/* Prepare device and return */
	return pcm_play_prepare();
}

int pcm_play(char *buffer, int length)
{	
	int err;

	if ((err = snd_pcm_writei (playback_handle, buffer, length)) != length) {
		//fprintf (stderr, "write to audio interface failed (%s)\n",
		//	snd_strerror (err));

		switch(err) {
			case -EBADFD:
				perror("snd_pcm_writei() error:  PCM is not in the right state.\n");
				break;
			case -EPIPE:
				perror("snd_pcm_writei() error:  An underrun occurred.\n");
				break;
			case -ESTRPIPE:
				perror("snd_pcm_writei() error:  A suspend event occurred (stream is suspended and waiting for an application recovery)\n");
				break;

			default:
				perror("snd_pcm_writei() error:  Unknown error code!\n");

		}

		return err;
	}
	return 0;
}

int pcm_play_prepare(void)
{
	int err;

	if ((err = snd_pcm_prepare (playback_handle)) < 0) {
		fprintf (stderr, "cannot prepare audio interface for use (%s)\n",
			 snd_strerror (err));

		return err;
	}
	return 0;
}

int pcm_play_close()
{
	snd_pcm_close (playback_handle);
	return 0;
}
