/* A simple loopback program for the GPP side of a server-client
	tcp socket setup
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "tcplib.h"
#include "logger.h"

#define BUFF_LEN 	320

enum imbe_ctrl_t {
	IMBE_CTRL_FAIL,
	IMBE_DATA,
	IMBE_ENCODE_START,
	IMBE_ENCODE_STOP,
	IMBE_DECODE_START,
	IMBE_DECODE_STOP
};

int sockfd, connfd;

static int started = 0;
void loop_shutdown(int signum)
{
        INFO("Gracefully shutting down...\n");
        started = 0;

	// Close connection and TCP server
	close_tcp_server_connection(connfd);
	stop_tcp_server(sockfd);

	exit(signum);
}

void print_usage(char *execname)
{
	printf("usage:  %s <port>\n", execname);
	printf("        <port>     TCP server listening port [1025-65534]\n\n");
}

enum imbe_ctrl_t read_ctrl_pkt(void)
{
	int res;
	enum imbe_ctrl_t ctrl;

	res = read_tcp_server_stream(connfd, (char *)&ctrl, sizeof(int));
	if (res < 0) {
		ERROR("ERROR, reading from host ");
		return IMBE_CTRL_FAIL;
	}
	else if (res == 0) {
		INFO("Client disconnected.\n");
		return IMBE_CTRL_FAIL;
	}

	return ctrl;
}

int read_data_payload(char *buf, int len)
{
	int res;

	res = read_tcp_server_stream(connfd, buf, len);
	if (res < 0) {
		ERROR("ERROR, reading from host ");		
		return -1;
	}
	else if (res == 0) {
		INFO("Client disconnected.\n");
		return -1;
	}

	return res;
}

int main(int argc, char **argv) 
{
    	int res, port, host_connected;
	enum imbe_ctrl_t ctrl;
	char buf[BUFF_LEN];

	// Print usage help message if no command-line arguments specified
	if (argc == 1) {
		print_usage(argv[0]);
		exit(0);
	}

	// Read in TCP listening port from command-line parameters
	sscanf(argv[1], "%d", &port);
	if (port < 1025 || port > 65534) {
		ERROR("ERROR, not a valid port ");
		exit(1);
	}
	
        // Setup termination signal interrupt handler
        signal((int) SIGINT, loop_shutdown);

	started = 1;
	while (started) {
		// Start TCP server, listen for connections
		sockfd = start_tcp_server(port);
		INFO("Waiting for connection on port %d [sock = %d]\n", port, sockfd);
		do {
			connfd = get_tcp_server_connection(sockfd);
			usleep(100);
		} while (!connfd);
		host_connected = 1;

		// Client connected, start main processing loop
		while (host_connected) {
			// Read in control packet to determine state
			ctrl = read_ctrl_pkt();
			if (ctrl == IMBE_CTRL_FAIL) {
				ERROR("read_ctrl_pkt() failure!\n");
				host_connected = 0;
				break;
			}

			// Only valid encoding states are IMBE_ENCODE_START and IMBE_DECODE_START.
			if ((ctrl == IMBE_ENCODE_START) || (ctrl = IMBE_DECODE_START)) {				
				if (ctrl == IMBE_ENCODE_START) {
					// TODO:  set run encoder option struct
				}
				else {
					// TODO:  set run decoder option struct
				}

				// Start processing data until stop encode/decode control signal received.
				do {
					// Read in control packet to determine state
					ctrl = read_ctrl_pkt();
					if (ctrl == IMBE_CTRL_FAIL) {
						ERROR("read_ctrl_pkt() failure!\n");
						host_connected = 0;
						break;
					}

					// If control state is IMBE_DATA, retrieve data payload
					else if (ctrl == IMBE_DATA) {
						//INFO("IMBE_DATA\n");

						// Read in data payload since were are in Data Encode mode.
						res = read_data_payload(buf, sizeof(buf));
						if (res == -1) {
							ERROR("read_data_payload() failure!\n");
							host_connected = 0;
							break;
						}
					
						// AMBE encode/decode data!




						// Send data back to host!
						

						
					}				
				} while (!((ctrl == IMBE_ENCODE_STOP) || (ctrl == IMBE_DECODE_STOP)));
			}
		}

		// Close connection and TCP server
		close_tcp_server_connection(connfd);
		stop_tcp_server(sockfd);
	}

	return 0;
}



