/* A simple loopback program for the host side of a server-client 
	tcp socket setup
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <alsa/asoundlib.h>
#include "alsa_capture.h"
#include "alsa_play.h"
#include "tcplib.h"
#include "logger.h"
#include "ticktock.h"

// Length of buffer in bytes
#define BUFF_LEN  	320

// Length used for PCM sampling/playback - each sample is 16 bits long
#define S_BUFF_LEN	BUFF_LEN / 2

// PCM audio sampling rate
#define SAMPRATE	8000

int connfd;

enum imbe_ctrl_t {
        SOCK_ERROR,
        IMBE_DATA,
        IMBE_ENCODE_START,
        IMBE_ENCODE_STOP,
        IMBE_DECODE_START,
	IMBE_DECODE_STOP
};

struct host_pkt {
        enum imbe_ctrl_t 	ctrl;
        char 			payload[BUFF_LEN];
};


static int started = 0;
void loop_shutdown(int signum)
{
	INFO("Gracefully shutting down...ERRoR@)#**#@(!~!({\n");
	started = 0;

	// Close TCP client connection and shutdown PCM capture and playback
	stop_tcp_client(connfd);

	exit(signum);
}


void print_usage(char *execname)
{
        printf("usage:  %s <ip> <port>\n", execname);
        printf("        <ip>       TCP server IP address\n");
        printf("        <port>     TCP server port [1025-65534]\n\n");
}

void *listener_thread(void *arg)
{
	int res;
	char rec_buff[BUFF_LEN];

	FILE *testfile_enc = fopen("audio_enc.dat","w");
	if (testfile_enc == NULL)
		printf("Can't open file to output IMBE-encoded PCM data\n"); 

	while (started)
	{
		/////  Read IMBE encoded PCM data from TCP socket ////////////
		if ((res = read_tcp_server_stream(connfd, rec_buff, sizeof(rec_buff))) < 0)
			ERROR("ERROR reading from gpp"); 

		////   Write received IMBE-encoded PCM data to file ////////////
		if (fwrite(rec_buff, sizeof(char), res, testfile_enc) < 0)
			printf("Error writing IMBE-encoded PCM data to file\n");
			
		fflush(testfile_enc);

	}

	fclose(testfile_enc);

	return NULL;
}


int main(int argc, char **argv)
{
	int res, port, count = 0;	
	struct host_pkt pkt;
    	pthread_attr_t attr;
	pthread_t lThread;

	char *gpp_ip_address; 
	char *snd_cap_device = "plughw:1,0",    *snd_play_device = "plughw:1,0";
	
	FILE *testfile;
	if ((testfile = fopen("audio.dat","r")) == NULL)
		printf("Can't open file to read PCM data\n");

	// Print out usage message if command-line argument length not sufficient.
        if (argc < 3) {
                print_usage(argv[0]);
                exit(0);
        }


	// Obtain IP address and port from command-line parameters
	gpp_ip_address = argv[1];
	res = sscanf(argv[2], "%d", &port);
	if (port < 1025 || port > 65534)
		ERROR("Not a valid port!\n");

	// Setup termination signal interrupt handler
        signal((int) SIGINT, loop_shutdown);
	
	// Start TCP connection to GPP server
	printf("Init TCP connection to %s:%d...", gpp_ip_address, port);	
	connfd = start_tcp_client(gpp_ip_address, port);
	printf("done.\n");

	// Open up PCM capture channel on sound device
	printf("Init PCM capture on sound device:  %s...", snd_cap_device);
	pcm_cap_init(snd_cap_device, SAMPRATE);
	printf("done.\n");

	// Open up PCM playback channel on sound device
	printf("Init PCM playback on sound device:  %s...", snd_cap_device);
	pcm_play_init(snd_play_device, SAMPRATE);
	printf("done.\n");

	// Start TCP listener thread
	started = 1;
    	pthread_attr_init(&attr);
        pthread_attr_setschedpolicy(&attr, SCHED_FIFO);
        pthread_attr_setstacksize(&attr, 131072);
    	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
    	res = pthread_create(&lThread, &attr, listener_thread, NULL);
    	if (res) ERROR("Failed to create TCP listener thread");
    	pthread_attr_destroy(&attr);

	pkt.ctrl = IMBE_ENCODE_START;
	if (send_tcp_client_data(connfd, (char *)&pkt.ctrl, sizeof(pkt.ctrl)) < 0)
		printf("ERROR writing control start to gpp\n");
	
	count = 0;
	do {
//		TICK();

		pkt.ctrl = IMBE_DATA;

/*		////////  Capture from audio hardware //////////
		while (pcm_capture(pkt.payload, S_BUFF_LEN)  < 0) {
			pcm_capture_prepare();
			ERROR("Buffer Overrun!!!\n");
		}
*/		
		////// Read from a file for testing /////////////
		if ((res = fread(pkt.payload, sizeof(char), BUFF_LEN, testfile)) < 0) {
			ERROR("Error reading PCM data from file\n");
		}
		else if (res == 0) {
			INFO("EOF reached, file read complete.\n");
			break;
		}
		
		/////  Send to tcp socket ///////////
		if (send_tcp_client_data(connfd, (char *)&pkt, sizeof(struct host_pkt)) <0)    
			ERROR("ERROR writing payload to gpp\n");
/*		
		//////  Play to audio hardware  //////////////
		while (pcm_play(rec_buff, S_BUFF_LEN) < 0) {
			pcm_play_prepare();
			ERROR("Buffer Underrun!!!\n");
		}
*/

//		TOCK("full loop");
		INFO("count = %d\n", count++);
	} while (started);

	
	pkt.ctrl = IMBE_ENCODE_STOP;
	if (send_tcp_client_data(connfd, (char *)&pkt.ctrl, sizeof(pkt.ctrl)) < 0)
		printf("ERROR writing control stop to gpp\n");
	
	if ((fclose(testfile)) < 0)
		printf("Error closing PCM file\n");

	// Close sound card handles
	pcm_play_close();
	pcm_cap_close();
	// Close tcp socket
	if (stop_tcp_client(connfd) < 0)
		printf("ERROR closing client\n");

	return 0;
}





