/** ============================================================================
 *  @file   dprintf.c
 *  @path   $(DSPLINK)/dsp/src/samples/dsp_ambe/
 *  @desc   Main function that calls SWI or TSK dsp_ambeback applications based
 *          on the compilation flag SWI_MODE or TSK_MODE.
 *  @ver    1.65.00.03
 *  ============================================================================
 *  ============================================================================
 */

/*  ----------------------------------- OS Specific Headers           */
#include <stdio.h>

/*  ----------------------------------- DSP/BIOS LINK Header       */
#include <dsplink.h>

/*  ----------------------------------- DSP/BIOS LINK API               */
#include <proc.h>
#include <notify.h>

/*  ----------------------------------- SAMPLE Headers		*/
#include "dprintf.h"
#include "log_arm.h"

Uint8 DPRINTF_dspId;

/** ----------------------------------------------------------------------------
 *  @func   dprintf_init_gpp
 *  @desc   Initialize dprintf for the ARM side.
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
int dprintf_init_gpp (unsigned char processorId) 
{
    DSP_STATUS status = DSP_SOK;

	DPRINTF_dspId = processorId;

	// register callback from the dsp side: 
	status = NOTIFY_register (DPRINTF_dspId, DPRINTF_IPS_ID, DPRINTF_IPS_EVENTNO, 
		(FnNotifyCbck) dprintf_gpp_callback, (Uint32) 0); 
	return status;
}


/** ----------------------------------------------------------------------------
 *  @func   dprintf_gpp_callback
 *  @desc   Callback function for dprintf on the ARM side.
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
void dprintf_gpp_callback (unsigned long int eventNo, void * arg, void * info)
{ 
	static char PrintBuffer[DPRINTF_BUFFERSZ]; 
	DSP_STATUS status = DSP_SOK;

	// get data from DSP: 
	status = PROC_read(0, (int)info, DPRINTF_BUFFERSZ, PrintBuffer); 
	if (DSP_SUCCEEDED(status)) {
		printf("DSP: %s", PrintBuffer);
		fflush(stdout);
	} else 
		ERROR("DEBUG: Unable to read dsp-mem %p\n", info); 

	// notify the DSP-side: 
	status = NOTIFY_notify (DPRINTF_dspId, DPRINTF_IPS_ID, DPRINTF_IPS_EVENTNO, (Uint32) 0); 
	if (DSP_FAILED(status))
		ERROR("NOTIFY_notify() failed. Status = [0x%x]\n",(int)status); 
} 
